library(tm)
library(lda)
library(topicmodels)
library(ROCR)
library(parallel)

### console --------------------------------------------------------------------
options(max.print = 100)

### load data ------------------------------------------------------------------
data.corpus <- readRDS("./saved/corpus.rds")
data <- data.frame(
  meta(data.corpus, "label", type="indexed"),
  meta(data.corpus, "file",  type="indexed")
)
data.dtm <- readRDS("./saved/dtmatrix.rds")
data.dtm <- dtm2ldaformat(data.dtm, omit_empty = FALSE)

### training & testing datasets ------------------------------------------------
trainset.files <- c(
    "T6_2013_Queensland_Floods"   # 8732
  , "T6_2013_Alberta_Floods"      # 9069
  , "T26_2013_Manila_floods"      # 1167
  , "T26_2013_Sardinia_floods"    #  180
  , "T26_2012_Philipinnes_floods" # 1318
)
testset.files <- c(
    "T26_2013_Colorado_floods"    # 1664
)
trainset.idx <- which(data$file %in% trainset.files)
testset.idx  <- which(data$file %in% testset.files)

cat(sprintf("train/test: %d / %d (%.1f%% / %.1f%%)",
            length(trainset.idx), length(testset.idx),
            length(trainset.idx)/(length(trainset.idx)+length(testset.idx))*100,
            length(testset.idx) /(length(trainset.idx)+length(testset.idx))*100
))

### sLDA -----------------------------------------------------------------------

trainset.size <- seq(1000, 20000, 1000)
trainset.size <- rep(trainset.size, 3)

model.quality <- mclapply(trainset.size, mc.cores = 3, FUN = function(iter.size) {
  # choose raindom records for the trainset of demanding size
  # iter.size <- 1000
  iter.idx <- sample(trainset.idx, iter.size, replace = FALSE)

  # train and test model -------------------------------------------------------
  topics.cnt <- 100
  model <- slda.em(
    documents = data.dtm$documents[iter.idx],
    K = topics.cnt,
    vocab = data.dtm$vocab,
    num.e.iterations = 10,
    num.m.iterations =  4,
    alpha = 1.0, eta = 0.1,
    annotations = data$label[iter.idx],
    params = sample(c(0,1), topics.cnt, replace = TRUE),
    variance = 0.25,
    logistic = FALSE,    # default
    lambda = 1.0,
    regularise = FALSE,  # default
    method="sLDA",
    trace = 0L           # default
  )

  predict <- slda.predict(
    documents = data.dtm$documents[testset.idx],
    topics = model$topics,
    model = model$model,
    alpha = 1.0,
    eta = 0.1,
    num.iterations = 1000,   # default
    average.iterations = 50, # default
    trace = 0L               # default 0L
  )

  # accuracy metrics -----------------------------------------------------------
  rocr.pred <- prediction(
    predictions = predict,
    labels      = data$label[testset.idx]
  )
  rocr.perf    <- performance(rocr.pred, measure = "acc", x.measure = "cutoff")
  cutoff.index <- which.max(rocr.perf@y.values[[1]])

  return(c(
    "prec" = performance(rocr.pred, measure = "prec")@y.values[[1]][cutoff.index],
    "rec"  = performance(rocr.pred, measure = "rec" )@y.values[[1]][cutoff.index],
    "auc"  = performance(rocr.pred, measure = "auc" )@y.values[[1]]
  ))
})

model.quality <- do.call(rbind, model.quality)
model.quality <- as.data.frame.matrix(model.quality)
model.quality$size <- trainset.size

if (file.exists("./saved/training_size_slda.csv") == TRUE) {
  result <- read.csv("./saved/training_size_slda.csv", header = TRUE)
  result <- rbind(result, model.quality)
} else {
  result <- model.quality
}
write.csv(result, file = "./saved/training_size_slda.csv", row.names = FALSE)

### ----------------------------------------------------------------------------

rm(
  data, data.corpus, data.dtm,
  testset.idx, testset.files, trainset.idx, trainset.files, trainset.size,
  model.quality, result
)
