library(tm)
library(RTextTools)
library(cldr)
library(slam)

### load model and variables ---------------------------------------------------
vocabulary <- readRDS("./saved/vocabulary.rds")
model      <- readRDS("./saved/model.final.rds")

# ---- custom functions to clean corpora
removeRegEx <- content_transformer( function(x, pattern) { gsub(pattern, "", x) } )
regex.user  <- "(@[a-z0-9_]*)|(#[a-z0-9_]*)" # users and hash-tags
regex.url   <- "(?:https?://|www.|ftp.)[-a-z0-9+&@#/%=~_|$?!:,.…]*[a-z0-9+&@#/%=~_|$…]"
regex.html  <- "&[a-z]+;"                    # html artefacts: "&amp;" "&lt;"


data.classify <- function(data) {
  # ---- create corpus
  data.corpus <- VCorpus(VectorSource(data$message))
  meta(data.corpus, tag = "hashtags", type="indexed") <- data$hashtags
  data.corpus <- tm_map(data.corpus, content_transformer(tolower), lazy=TRUE)
  data.corpus <- tm_map(data.corpus, removeRegEx, regex.user)
  data.corpus <- tm_map(data.corpus, removeRegEx, regex.url )
  data.corpus <- tm_map(data.corpus, removeRegEx, regex.html)
  data.corpus <- tm_map(data.corpus, removePunctuation)
  # ---- check language (package `cldr`)
  data.corpus <- tm_filter(data.corpus, FUN = function(doc) {
    lang <- detectLanguage(
      content(doc),
      hintLanguageCode = Languages$ENGLISH, isPlainText = TRUE
    )
    return(lang$detectedLanguage == "ENGLISH")
  })
  data.corpus <- tm_map(data.corpus, removeWords, stopwords("english"), lazy=TRUE)
  data.corpus <- tm_map(data.corpus, stemDocument, language = "english")
  # ---- dt-matrix
  data.dtm  <- DocumentTermMatrix(data.corpus, control = list(dictionary = vocabulary))
  rowTotals <- slam::row_sums(data.dtm) # empty rows
  if (nrow(data.dtm[rowTotals > 0, ]) == 0) {
    # dt-matrix is empty
    data.predict <- rep(FALSE, times = nrow(data))
    return(data.predict)
  }
  # ---- classify
  model.container <- create_container(
    matrix   = data.dtm,
    labels   = sample(c(0,1), size=data.dtm$nrow, replace=TRUE), # random labels
    testSize = 1:data.dtm$nrow,
    virgin   = TRUE
  )
  model.predict <- classify_model(model.container, model)
  levels(model.predict$SVM_LABEL) <- c("FALSE", "TRUE")
  # ---- set labels of the removed records to `NULL` and propagate classifed
  data.predict <- rep(NA, times = nrow(data))
  id <- meta(data.corpus, tag = "id")
  id <- as.numeric(id)
  data.predict[id] <- as.logical(model.predict$SVM_LABEL)
  
  return(data.predict)
}
